﻿using System.Collections.Generic;
using Naturalis.BM.UI;
using Naturalis.Common;

namespace Naturalis.BM
{
    sealed class Bindings : BindingsBase
    {
        public new void Init(ScriptBase script, string namespaceName)
        {
            base.Init(script, namespaceName);
            AddActions(new List<JSONStorableAction>
            {
                script.NewJSONStorableAction(Constant.OPEN_UI, OpenUI),
                script.NewJSONStorableAction($"{Constant.OPEN_UI}_{Constant.MAIN}", OpenUIMain),
                script.NewJSONStorableAction($"{Constant.OPEN_UI}_{Constant.FRICTION}", OpenUIFriction),
                script.NewJSONStorableAction($"{Constant.OPEN_UI}_{Constant.PHYSICS}", OpenUIPhysics),
                script.NewJSONStorableAction($"{Constant.OPEN_UI}_{Constant.MORPHS}", OpenUIMorphs),
                script.NewJSONStorableAction($"{Constant.OPEN_UI}_{Constant.HARD_COLLIDERS}", OpenUIHardColliders),
                new JSONStorableAction("Calibrate", () => BootyMagic.script.StartCalibration()),
            });
            if(Settings.envIsDevelopment)
            {
                AddActions(new List<JSONStorableAction>
                {
                    new JSONStorableAction($"{Constant.OPEN_UI}_Dev", OpenUIDev),
                    new JSONStorableAction($"{Constant.OPEN_UI}_DevMorph", OpenUIDevMorph),
                });
            }
        }

        void OpenUIMain()
        {
            StartCoroutine(SelectPluginUI(BootyMagic.tabs.GoToMainWindow));
        }

        void OpenUIPhysics()
        {
            StartCoroutine(SelectPluginUI(BootyMagic.tabs.GoToPhysicsWindow));
        }

        void OpenUIMorphs()
        {
            StartCoroutine(SelectPluginUI(BootyMagic.tabs.GoToMorphsWindow));
        }

        void OpenUIHardColliders()
        {
            StartCoroutine(SelectPluginUI(BootyMagic.tabs.GoToHardCollidersWindow));
        }

        void OpenUIFriction()
        {
            if(script.enabled)
            {
                StartCoroutine(
                    SelectContainingAtomTab(
                        "Skin Materials 2",
                        () => BootyMagic.script.uiModManager.switchToBootyMagicAction.actionCallback()
                    )
                );
            }
            else
            {
                StartCoroutine(SelectContainingAtomTab("Plugins"));
            }
        }

        void OpenUIDev()
        {
            OpenUIMain();
            OpenNestedWindow<DevWindow>(BootyMagic.tabs.mainWindow, BootyMagic.tabs.mainWindow.openDevWindowAction);
        }

        void OpenUIDevMorph()
        {
            OpenUIMain();
            OpenNestedWindow<DevMorphWindow>(BootyMagic.tabs.mainWindow, BootyMagic.tabs.mainWindow.openDevMorphWindowAction);
        }

        static void OpenNestedWindow<T>(MainWindow mainWindow, JSONStorableAction openAction)
        {
            var nestedWindow = mainWindow.GetActiveNestedWindow();
            if(nestedWindow is T)
            {
                return;
            }

            if(nestedWindow != null)
            {
                nestedWindow.Clear();
            }

            openAction.actionCallback();
        }
    }
}
