﻿using System.Collections.Generic;
using Naturalis.BM.Components;
using Naturalis.Common;
using Naturalis.Common.Components;
using Naturalis.Common.Models;
using UnityEngine;
using static Direction;

namespace Naturalis.BM.Handlers
{
    static class ForceMorphHandler
    {
        static Glute _leftGlute;
        static Glute _rightGlute;

        public static Dictionary<string, List<ConfigurableMorph>> morphSets { get; private set; }

        public static StorableFloat baseJsf { get; private set; }
        public static StorableFloat upJsf { get; private set; }
        public static StorableFloat downJsf { get; private set; }
        public static StorableFloat depthInJsf { get; private set; }
        public static StorableFloat depthOutJsf { get; private set; }
        public static StorableFloat sidewaysInJsf { get; private set; }
        public static StorableFloat sidewaysOutJsf { get; private set; }

        static float UserUpMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * upJsf.val);
        }

        static float UserDownMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * downJsf.val);
        }

        static float UserDepthInMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * depthInJsf.val);
        }

        static float UserDepthOutMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * depthOutJsf.val);
        }

        static float UserSidewaysInMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * sidewaysInJsf.val);
        }

        static float UserSidewaysOutMultiplier()
        {
            return Calc.QuadraticRegression(baseJsf.val * sidewaysOutJsf.val);
        }

        public static JSONStorableStringChooser directionJssc { get; private set; }

        public static void Init()
        {
            _leftGlute = BootyMagic.script.leftGlute;
            _rightGlute = BootyMagic.script.rightGlute;

            baseJsf = BootyMagic.script.NewStorableFloat("Force Morphing Base Multiplier", 1.00f, 0.00f, 2.00f);
            upJsf = BootyMagic.script.NewStorableFloat("Force Morphing Up", 1.00f, 0.00f, 2.00f);
            downJsf = BootyMagic.script.NewStorableFloat("Force Morphing Down", 1.00f, 0.00f, 2.00f);
            depthInJsf = BootyMagic.script.NewStorableFloat("Force Morphing Depth In", 1.00f, 0.00f, 2.00f);
            depthOutJsf = BootyMagic.script.NewStorableFloat("Force Morphing Depth Out", 1.00f, 0.00f, 2.00f);
            sidewaysInJsf = BootyMagic.script.NewStorableFloat("Force Morphing Sideways In", 1.00f, 0.00f, 2.00f);
            sidewaysOutJsf = BootyMagic.script.NewStorableFloat("Force Morphing Sideways Out", 1.00f, 0.00f, 2.00f);

            baseJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            upJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            downJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            depthInJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            depthOutJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            sidewaysInJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;
            sidewaysOutJsf.setJSONCallbackFunction = OnMorphMultiplierValueChanged;

            LoadSettings();
        }

        static void OnMorphMultiplierValueChanged(JSONStorableFloat jsf)
        {
            if(SoftPhysicsHandler.LdpUpdateIsEnabled())
            {
                BootyMagic.script.SetCalibrationPending((StorableFloat) jsf);
            }
        }

        static void LoadSettings()
        {
            morphSets = new Dictionary<string, List<ConfigurableMorph>>
            {
                { UP_L, UpForceMorphs("L") },
                { UP_R, UpForceMorphs("R") },
                { DOWN_L, DownForceMorphs("L") },
                { DOWN_R, DownForceMorphs("R") },
                { BACK_L, DepthInForceMorphs("L") },
                { BACK_R, DepthInForceMorphs("R") },
                { FORWARD_L, DepthOutForceMorphs("L") },
                { FORWARD_R, DepthOutForceMorphs("R") },
                { LEFT_L, LeftForceLeftMorphs() },
                { LEFT_R, LeftForceRightMorphs() },
                { RIGHT_L, RightForceLeftMorphs() },
                { RIGHT_R, RightForceRightMorphs() },
            };

            var directions = new List<string>
            {
                UP,
                // UP_C,
                DOWN,
                BACK,
                // BACK_C,
                FORWARD,
                // FORWARD_C,
                LEFT_L,
                RIGHT_L,
                LEFT_R,
                RIGHT_R,
            };
            directionJssc = new JSONStorableStringChooser("direction", directions, directions[0], "Direction");
        }

#region *** Morph Configuration ***

        static ConfigurableMorph NewConfigurableMorph(string morphName, bool isNegative, float softMultiplier, float massMultiplier)
        {
            return new ConfigurableMorph(
                Person.GetGluteMorph(morphName),
                isNegative,
                new JSONStorableFloat("softMultiplier", softMultiplier, -3.00f, 3.00f),
                new JSONStorableFloat("massMultiplier", massMultiplier, -3.00f, 3.00f)
            );
        }

        static List<ConfigurableMorph> UpForceMorphs(string side)
        {
            const string prefix = MorphPrefix.UP;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Y {side}", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute UpDown {side}", true, -1.60f, 0.60f),
                NewConfigurableMorph($"{prefix} Glute Move Up {side}", false, 1.80f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Upper Spread {side}", false, 0.50f, -0.25f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", false, 0.60f, 0.40f),
            };
        }

        static List<ConfigurableMorph> DownForceMorphs(string side)
        {
            const string prefix = MorphPrefix.DN;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Y {side}", true, -0.50f, -1.00f),
                NewConfigurableMorph($"{prefix} Glute UpDown {side}", false, 0.00f, 0.00f),
                NewConfigurableMorph($"{prefix} Glute Move Down {side}", true, -0.15f, -0.30f),
                NewConfigurableMorph($"{prefix} Glute Height Lower {side}", true, -0.20f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Line {side}", false, 0.30f, 0.60f),
            };
        }

        static List<ConfigurableMorph> DepthInForceMorphs(string side)
        {
            const string prefix = MorphPrefix.BK;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Scale Z {side}", true, -0.90f, -1.20f),
                NewConfigurableMorph($"{prefix} Glute Height {side}", false, 1.00f, 1.80f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", false, 1.50f, 2.85f),
            };
        }

        static List<ConfigurableMorph> DepthOutForceMorphs(string side)
        {
            const string prefix = MorphPrefix.FW;
            return new List<ConfigurableMorph>
            {

                NewConfigurableMorph($"{prefix} Glute Scale Z {side}", false, 0.80f, 1.60f),
                NewConfigurableMorph($"{prefix} Glute Height {side}", true, -0.36f, -0.48f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", true, -0.50f, -0.67f),
            };
        }

        static List<ConfigurableMorph> LeftForceLeftMorphs()
        {
            const string prefix = MorphPrefix.LT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z L", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Move Out L", false, 1.00f, -0.20f),
            };
        }

        static List<ConfigurableMorph> LeftForceRightMorphs()
        {
            const string prefix = MorphPrefix.LT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z R", true, -0.45f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Move In R", false, 1.10f, -0.40f),
            };
        }

        static List<ConfigurableMorph> RightForceLeftMorphs()
        {
            const string prefix = MorphPrefix.RT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z L", true, -0.45f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Move In L", false, 1.10f, -0.40f),
            };
        }

        static List<ConfigurableMorph> RightForceRightMorphs()
        {
            const string prefix = MorphPrefix.RT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z R", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Move Out R", false, 1.00f, -0.20f),
            };
        }

#endregion

        static bool _softPhysicsOn;
        static float _mass;
        static float _softness;

        static float _upDownLimit;
        static float _depthInLimit;
        static float _depthOutLimit;
        static float _sidewaysLimit;

        static float _upMultiplier;
        static float _downMultiplier;
        static float _depthInMultiplier;
        static float _depthOutMultiplier;
        static float _sidewaysMultiplier;

        static float SoftnessMultiplier(float x)
        {
            return Calc.InverseSmoothStep(x, 1.00f, 0.80f, 0.03f);
        }

        public static void SetMultipliers()
        {
            _softPhysicsOn = SoftPhysicsHandler.softPhysicsOn;
            _mass = MainPhysicsHandler.normalizedRealMass;
            _softness = SoftnessMultiplier(BootyMagic.softness);

            SetUpDownMultipliers();
            SetDepthInMultiplier();
            SetDepthOutMultiplier();
            SetSidewaysMultiplier();
        }

        static void SetUpDownMultipliers()
        {
            _upDownLimit = 50 * Mathf.Lerp(1, 1.20f, _mass);
            float spMultiplier = _softPhysicsOn ? 1 : 0.67f;
            float softnessMultiplier = Mathf.Lerp(0.50f * spMultiplier, 1.25f * spMultiplier, _softness);
            _upMultiplier = softnessMultiplier * 1.50f * Mathf.Lerp(1, 1.80f, _mass);
            _downMultiplier = softnessMultiplier * 1.80f * Mathf.Lerp(1, 1.80f, _mass);
        }

        static void SetDepthInMultiplier()
        {
            _depthInLimit = 0.045f * Mathf.Lerp(1, 3.00f, _mass);
            float spMultiplier = _softPhysicsOn ? 1 : 0.55f;
            float softnessMultiplier = Mathf.Lerp(0.50f * spMultiplier, 1.00f * spMultiplier, _softness);
            _depthInMultiplier = softnessMultiplier * 4.40f * Mathf.Lerp(1, 0.50f, _mass);
        }

        static void SetDepthOutMultiplier()
        {
            _depthOutLimit = 0.045f * Mathf.Lerp(1, 3.00f, _mass);
            float spMultiplier = _softPhysicsOn ? 1 : 0.55f;
            float softnessMultiplier = Mathf.Lerp(0.65f * spMultiplier, 1.125f * spMultiplier, _softness);
            _depthOutMultiplier = softnessMultiplier * 2.20f * Mathf.Lerp(1, 1.50f, _mass);
        }

        static void SetSidewaysMultiplier()
        {
            _sidewaysLimit = 50 * Mathf.Lerp(1, 1.20f, _mass);
            float spMultiplier = _softPhysicsOn ? 1 : 0.67f;
            float softnessMultiplier = Mathf.Lerp(0.50f * spMultiplier, 1.25f * spMultiplier, _softness);
            _sidewaysMultiplier = softnessMultiplier * 1.10f * Mathf.Lerp(1, 1.80f, _mass);
        }

        public static void SaveCalibrated()
        {
            baseJsf.SaveCalibrated();
            upJsf.SaveCalibrated();
            depthInJsf.SaveCalibrated();
            depthOutJsf.SaveCalibrated();
            sidewaysInJsf.SaveCalibrated();
            sidewaysOutJsf.SaveCalibrated();
        }

        public static void Update()
        {
            AdjustUpMorphs();
            AdjustDownMorphs();
            AdjustDepthOutMorphs();
            AdjustDepthInMorphs();
            AdjustSidewaysMorphs();
        }

        static float UpDownForceEffectCurve(float x)
        {
            return Calc.Exponential1(x, 1.60f, 2.14f, 1.18f, 0.96f);
        }

        static float UpEffect(float angle)
        {
            return _upMultiplier *
                UserUpMultiplier() *
                UpDownForceEffectCurve(Mathf.Abs(angle) / _upDownLimit);
        }

        static void AdjustUpMorphs()
        {
            if(_leftGlute.angleY <= 0)
            {
                // up force on left glute
                UpdateMorphs(UP_L, UpEffect(_leftGlute.angleY));
            }
            else
            {
                // down force on left glute
                ResetMorphs(UP_L);
            }

            if(_rightGlute.angleY <= 0)
            {
                // up force on right glute
                UpdateMorphs(UP_R, UpEffect(_rightGlute.angleY));
            }
            else
            {
                // down force on right glute
                ResetMorphs(UP_R);
            }
        }

        static float DownEffect(float angle)
        {
            return _downMultiplier *
                UserDownMultiplier() *
                UpDownForceEffectCurve(Mathf.Abs(angle) / _upDownLimit);
        }

        static void AdjustDownMorphs()
        {
            if(_leftGlute.angleY <= 0)
            {
                // up force on left glute
                ResetMorphs(DOWN_L);
            }
            else
            {
                // down force on left glute
                UpdateMorphs(DOWN_L, DownEffect(_leftGlute.angleY));
            }

            if(_rightGlute.angleY <= 0)
            {
                // up force on right glute
                ResetMorphs(DOWN_R);
            }
            else
            {
                // down force on right glute
                UpdateMorphs(DOWN_R, DownEffect(_rightGlute.angleY));
            }
        }

        static float DepthForceEffectCurve(float x)
        {
            return Calc.Exponential1(x, 1.60f, 2.14f, 1.18f, 0.96f);
        }

        static float DepthInEffect(float depthDiff)
        {
            return _depthInMultiplier *
                UserDepthInMultiplier() *
                DepthForceEffectCurve(Mathf.Abs(depthDiff) / _depthInLimit);
        }

        static void AdjustDepthInMorphs()
        {
            if(_leftGlute.depthDiff >= 0)
            {
                // in force on left glute
                ResetMorphs(BACK_L);
            }
            else
            {
                // out force on left glute
                UpdateMorphs(BACK_L, DepthInEffect(_leftGlute.depthDiff));
            }

            if(_rightGlute.depthDiff >= 0)
            {
                // in force on right glute
                ResetMorphs(BACK_R);
            }
            else
            {
                // out force on right glute
                UpdateMorphs(BACK_R, DepthInEffect(_rightGlute.depthDiff));
            }
        }

        static float DepthOutEffect(float depthDiff)
        {
            return _depthOutMultiplier *
                UserDepthOutMultiplier() *
                DepthForceEffectCurve(Mathf.Abs(depthDiff) / _depthOutLimit);
        }

        static void AdjustDepthOutMorphs()
        {
            if(_leftGlute.depthDiff >= 0)
            {
                // in force on left glute
                UpdateMorphs(FORWARD_L, DepthOutEffect(_leftGlute.depthDiff));
            }
            else
            {
                // out force on left glute
                ResetMorphs(FORWARD_L);
            }

            if(_rightGlute.depthDiff >= 0)
            {
                // in force on right glute
                UpdateMorphs(FORWARD_R, DepthOutEffect(_rightGlute.depthDiff));
            }
            else
            {
                // out force on right glute
                ResetMorphs(FORWARD_R);
            }
        }

        static float SidewaysForceEffectCurve(float x)
        {
            return Calc.Exponential1(x, 1.60f, 2.14f, 1.18f, 0.96f);
        }

        static float SidewaysInEffect(float angle)
        {
            return _sidewaysMultiplier *
                UserSidewaysInMultiplier() *
                SidewaysForceEffectCurve(Mathf.Abs(angle) / _sidewaysLimit);
        }

        static float SidewaysOutEffect(float angle)
        {
            return _sidewaysMultiplier *
                UserSidewaysOutMultiplier() *
                SidewaysForceEffectCurve(Mathf.Abs(angle) / _sidewaysLimit);
        }

        static void AdjustSidewaysMorphs()
        {
            if(_leftGlute.angleX <= 0)
            {
                // right force on left glute
                ResetMorphs(LEFT_L);
                UpdateMorphs(RIGHT_L, SidewaysInEffect(_leftGlute.angleX));
            }
            else
            {
                // left force on left glute
                ResetMorphs(RIGHT_L);
                UpdateMorphs(LEFT_L, SidewaysOutEffect(_leftGlute.angleX));
            }

            if(_rightGlute.angleX <= 0)
            {
                // right force on right glute
                ResetMorphs(LEFT_R);
                UpdateMorphs(RIGHT_R, SidewaysOutEffect(_rightGlute.angleX));
            }
            else
            {
                // left force on right glute
                ResetMorphs(RIGHT_R);
                UpdateMorphs(LEFT_R, SidewaysInEffect(_rightGlute.angleX));
            }
        }

        static void UpdateMorphs(string direction, float effect)
        {
            float halfEffect = effect / 2;
            foreach(var configurableMorph in morphSets[direction])
            {
                configurableMorph.Update(halfEffect, _mass, 1);
            }
        }

        public static void ResetAll()
        {
            if(morphSets == null)
            {
                return;
            }

            foreach(var kvp in morphSets)
            {
                foreach(var configurableMorph in kvp.Value)
                {
                    configurableMorph.Reset();
                }
            }
        }

        static void ResetMorphs(string direction)
        {
            foreach(var configurableMorph in morphSets[direction])
            {
                configurableMorph.Reset();
            }
        }

        public static void Destroy()
        {
            _leftGlute = null;
            _rightGlute = null;
            morphSets = null;
            baseJsf = null;
            upJsf = null;
            downJsf = null;
            depthOutJsf = null;
            depthInJsf = null;
            sidewaysInJsf = null;
            sidewaysOutJsf = null;
            directionJssc = null;
        }
    }
}
