using Naturalis.BM.Handlers;
using Naturalis.Common.Models;

namespace Naturalis.BM.Models
{
    sealed class GluteSoftPhysicsParameterGroup : SoftPhysicsParameterGroup
    {
        public override void SetCallbackFunctions()
        {
            left.offsetJsf.setCallbackFunction = value =>
            {
                left.UpdateOffsetValue(value);
                right.UpdateOffsetValue(applyChangesOnlyToLeftParamJsb.val ? 0 : value);
                BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
                SoftPhysicsHandler.UpdateSoftJointsDistortion(true);
            };

            applyChangesOnlyToLeftParamJsb.setCallbackFunction = value =>
            {
                /* Offset */
                right.UpdateOffsetValue(value ? right.offsetJsf.defaultVal : left.offsetJsf.val);

                /* Gradient */
                var lGradient = left.gradientConfig;
                var rGradient = right.gradientConfig;
                rGradient.outerMultiplierJsf.val =
                    value ? rGradient.outerMultiplierJsf.defaultVal : lGradient.outerMultiplierJsf.val;
                rGradient.centerMultiplierJsf.val =
                    value ? rGradient.centerMultiplierJsf.defaultVal : lGradient.centerMultiplierJsf.val;
                rGradient.middleShiftJsf.val =
                    value ? rGradient.middleShiftJsf.defaultVal : lGradient.middleShiftJsf.val;
                right.Sync();

                /* Distortion */
                var lDistortion = left.ldpConfig;
                var rDistortion = right.ldpConfig;
                rDistortion.SetCompressionMaxAndMultiplier(
                    value ? rDistortion.compressionEffectJsf.defaultVal : lDistortion.compressionEffectJsf.val
                );
                rDistortion.SetCompressionSlope(
                    value ? rDistortion.compressionSensitivityJsf.defaultVal : lDistortion.compressionSensitivityJsf.val
                );
                rDistortion.SetStretchingMaxAndMultiplier(
                    value ? rDistortion.stretchingEffectJsf.defaultVal : lDistortion.stretchingEffectJsf.val
                );
                rDistortion.SetStretchingSlope(
                    value ? rDistortion.stretchingSensitivityJsf.defaultVal : lDistortion.stretchingSensitivityJsf.val
                );
                SoftPhysicsHandler.UpdateSoftJointsDistortion(Side.RIGHT, true);
            };

            /* Gradient config storable callbacks */
            {
                var lGradient = left.gradientConfig;
                var rGradient = right.gradientConfig;
                lGradient.outerMultiplierJsf.setCallbackFunction = value =>
                    UpdateGradient(value, rGradient.outerMultiplierJsf);
                lGradient.centerMultiplierJsf.setCallbackFunction = value =>
                    UpdateGradient(value, rGradient.centerMultiplierJsf);
                lGradient.middleShiftJsf.setCallbackFunction = value =>
                    UpdateGradient(value, rGradient.middleShiftJsf);
            }

            /* Distortion config storable callbacks */
            {
                left.ldpConfig.compressionEffectJsf.setCallbackFunction = value =>
                {
                    SetCompressionMaxAndMultiplier(value);
                    SoftPhysicsHandler.UpdateSoftJointsDistortion(true);
                    BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
                };
                left.ldpConfig.compressionSensitivityJsf.setCallbackFunction = value =>
                {
                    SetCompressionSlope(value);
                    SoftPhysicsHandler.UpdateSoftJointsDistortion(true);
                    BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
                };
                left.ldpConfig.stretchingEffectJsf.setCallbackFunction = value =>
                {
                    SetStretchingMaxAndMultiplier(value);
                    SoftPhysicsHandler.UpdateSoftJointsDistortion(true);
                    BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
                };
                left.ldpConfig.stretchingSensitivityJsf.setCallbackFunction = value =>
                {
                    SetStretchingSlope(value);
                    SoftPhysicsHandler.UpdateSoftJointsDistortion(true);
                    BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
                };
                SetCompressionMaxAndMultiplier(left.ldpConfig.compressionEffectJsf.val);
                SetCompressionSlope(left.ldpConfig.compressionSensitivityJsf.val);
                SetStretchingMaxAndMultiplier(left.ldpConfig.stretchingEffectJsf.val);
                SetStretchingSlope(left.ldpConfig.stretchingSensitivityJsf.val);
            }
        }

        new void UpdateGradient(float value, JSONStorableFloat rightJsf)
        {
            base.UpdateGradient(value, rightJsf);
            BootyMagic.tabs.physicsWindow.OnSoftParamAdjusted(this);
        }
    }
}
