using System;
using System.Collections.Generic;
using System.Linq;
using Naturalis.BM.Handlers;
using Naturalis.Common;
using Naturalis.Common.UI;
using UnityEngine;
using UnityEngine.Events;

namespace Naturalis.BM.UI
{
    sealed class DevMorphWindow : WindowBase, IWindow
    {
        readonly Dictionary<string, UIDynamic> _sectionElements = new Dictionary<string, UIDynamic>();

        public DevMorphWindow(UnityAction onReturnToParent = null) : base(BootyMagic.script, nameof(DevMorphWindow), onReturnToParent)
        {
        }

        static string DirectionKey(string value)
        {
            return value.EndsWith("_C") || value.StartsWith(Direction.LEFT) || value.StartsWith(Direction.RIGHT)
                ? value
                : $"{value}_L";
        }

        protected override void OnBuild()
        {
            BuildLeftSide();
            BuildRightSide();
            RebuildSection(ForceMorphHandler.directionJssc.val);
        }

        void BuildLeftSide(bool rightSide = false)
        {
            AddElement(() =>
            {
                var button = script.CreateButton("Print", rightSide);
                button.button.onClick.AddListener(
                    () =>
                    {
                        var configs = ForceMorphHandler.morphSets[DirectionKey(ForceMorphHandler.directionJssc.val)];
                        string[] arr = configs.Select(config => config.ToCodeString(nameof(Person.GetGluteMorph))).ToArray();
                        Debug.Log(string.Join("", arr));
                    }
                );
                button.height = 55;
                return button;
            });

            AddElement(() =>
            {
                var button = script.CreateButton("Zero All", rightSide);
                button.button.onClick.AddListener(
                    () =>
                    {
                        var configs = ForceMorphHandler.morphSets[DirectionKey(ForceMorphHandler.directionJssc.val)];
                        foreach(var config in configs)
                        {
                            config.massMultiplierJsf.val = 0;
                            config.softMultiplierJsf.val = 0;
                        }
                    }
                );
                button.height = 55;
                return button;
            });
        }

        void BuildRightSide(bool rightSide = true)
        {
            AddSpacer(55, rightSide);
            AddElement(ForceMorphHandler.directionJssc.name, () =>
            {
                var popup = script.CreatePopupAuto(ForceMorphHandler.directionJssc, rightSide, 360f);
                popup.popup.labelText.color = Color.black;
                popup.popup.onValueChangeHandlers += RebuildSection;
                return popup;
            });
        }

        void RebuildSection(string value)
        {
            ClearSection();
            var leftConfigs = ForceMorphHandler.morphSets[DirectionKey(value)];
            foreach(var config in leftConfigs)
            {
                AddSectionElement($"{config.morphName} Label", () =>
                    CreateValueTextField(config.displayValueJss, 28, 50, false)
                );

                AddSectionElement($"{config.morphName} {config.isNegativeJsb.name}", () =>
                {
                    var toggle = script.CreateToggle(config.isNegativeJsb, true);
                    toggle.label = "Is Negative";
                    return toggle;
                });

                AddSectionElement($"{config.morphName} {config.softMultiplierJsf.name}", () =>
                {
                    var slider = script.CreateSlider(config.softMultiplierJsf);
                    slider.valueFormat = "F2";
                    slider.label = "Softness Multiplier";
                    return slider;
                });

                AddSectionElement($"{config.morphName} {config.massMultiplierJsf.name}", () =>
                {
                    var slider = script.CreateSlider(config.massMultiplierJsf, true);
                    slider.valueFormat = "F2";
                    slider.label = "Mass Multiplier";
                    return slider;
                });
            }
        }

        void AddSectionElement(string key, Func<UIDynamic> createElement)
        {
            _sectionElements[key] = createElement();
        }

        public new void Clear()
        {
            base.Clear();
            ClearSection();
            RemoveDirectionPopupChangeHandler();
        }

        void RemoveDirectionPopupChangeHandler()
        {
            var popup = GetElementAs<UIDynamicPopup>(ForceMorphHandler.directionJssc.name);
            if(popup)
            {
                popup.popup.visible = false;
                popup.popup.onValueChangeHandlers -= RebuildSection;
            }
        }

        void ClearSection()
        {
            foreach(var kvp in _sectionElements)
            {
                script.RemoveElement(kvp.Value);
            }

            _sectionElements.Clear();
        }
    }
}
