using System.Linq;
using Naturalis.BM.Handlers;
using Naturalis.Common.UI;
using UnityEngine;
using UnityEngine.UI;

namespace Naturalis.BM.UI
{
    sealed class MorphsWindow : WindowBase
    {
        public MorphsWindow() : base(BootyMagic.script, nameof(MorphsWindow))
        {
        }

        protected override void OnBuild()
        {
            BuildLeftSide();
            BuildRightSide();

            GetElement(ForceMorphHandler.baseJsf.name).AddListener(UpdateAllSliderColors);
            UpdateAllSliderColors(0);
        }

        void BuildLeftSide(bool rightSide = false)
        {
            AddHeaderTextField("Directional Force Morphing", rightSide);

            AddSpacer(5, rightSide);
            AddElement(ForceMorphHandler.baseJsf.name, () =>
            {
                var slider = script.CreateSlider(ForceMorphHandler.baseJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Base Multiplier";
                return slider;
            });

            AddInfoTextField(
                "Control how much glute shape is adjusted dynamically with morphs." +
                "\n\nThe amount of morphing is based on the glute's XY rotation and" +
                " Z position relative to the calibration pose. " +
                "\n\nAnything that causes the glute to move will cause morphing: collision, gravity" +
                " or any kind of animation.",
                rightSide,
                620,
                28
            );
        }

        void BuildRightSide(bool rightSide = true)
        {
            AddSpacer(80, rightSide);
            AddMultiplierSlider(ForceMorphHandler.upJsf, "Up", rightSide);
            AddMultiplierSlider(ForceMorphHandler.downJsf, "Down", rightSide);
            AddMultiplierSlider(ForceMorphHandler.depthOutJsf, "Depth Out", rightSide);
            AddMultiplierSlider(ForceMorphHandler.depthInJsf, "Depth In", rightSide);
            AddMultiplierSlider(ForceMorphHandler.sidewaysInJsf, "Sideways In", rightSide);
            AddMultiplierSlider(ForceMorphHandler.sidewaysOutJsf, "Sideways Out", rightSide);
        }

        void AddMultiplierSlider(JSONStorableFloat storable, string label, bool rightSide)
        {
            AddElement(storable.name, () =>
            {
                var slider = script.CreateSlider(storable, rightSide);
                slider.valueFormat = "F2";
                slider.label = label;
                slider.AddListener((float _) => UpdateSliderColor(storable));
                return slider;
            });
        }

        void UpdateAllSliderColors(float _)
        {
            UpdateSliderColor(ForceMorphHandler.upJsf);
            UpdateSliderColor(ForceMorphHandler.downJsf);
            UpdateSliderColor(ForceMorphHandler.depthOutJsf);
            UpdateSliderColor(ForceMorphHandler.depthInJsf);
            UpdateSliderColor(ForceMorphHandler.sidewaysInJsf);
            UpdateSliderColor(ForceMorphHandler.sidewaysOutJsf);
        }

        void UpdateSliderColor(JSONStorableFloat storable)
        {
            var slider = GetElementAs<UIDynamicSlider>(storable.name);
            if(slider)
            {
                var images = slider.slider.gameObject.transform.GetComponentsInChildren<Image>();
                var fillImage = images.First(image => image.name == "Fill");
                var handleImage = images.First(image => image.name == "Handle");
                var color = MultiplierSliderColor(ForceMorphHandler.baseJsf.val * storable.val);
                fillImage.color = color;
                handleImage.color = color;
            }
        }

        static Color MultiplierSliderColor(float value)
        {
            return value <= 1
                ? Color.Lerp(new Color(1, 1, 1, 0.25f), Color.white, value)
                : Color.Lerp(Color.white, new Color(1.0f, 0.2f, 0.2f), (value - 1) / 3);
        }
    }
}
