﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Naturalis.Common
{
    class BindingsBase : MonoBehaviour
    {
        protected ScriptBase script;
        public Dictionary<string, string> @namespace { get; private set; }
        public Dictionary<string, JSONStorableAction> actions { get; private set; }

        protected void Init(ScriptBase script, string namespaceName)
        {
            this.script = script;
            @namespace = new Dictionary<string, string>
            {
                { "Namespace", namespaceName },
            };
            actions = new Dictionary<string, JSONStorableAction>();
        }

        public IEnumerable<object> GetActionsList()
        {
            return actions.Values.Select(action => (object) action).ToList();
        }

        protected void AddActions(List<JSONStorableAction> actionsList)
        {
            foreach(var action in actionsList)
            {
                actions[action.name] = action;
            }
        }

        protected void OpenUI()
        {
            StartCoroutine(SelectPluginUI());
        }

        // adapted from Timeline v4.3.1 (c) acidbubbles
        protected IEnumerator SelectPluginUI(Action postAction = null)
        {
            while(script.initialized == null)
            {
                yield return null;
            }

            if(script.initialized == false)
            {
                yield break;
            }

            if(script.UITransform && script.UITransform.gameObject.activeInHierarchy)
            {
                if(script.enabled)
                {
                    postAction?.Invoke();
                }

                yield break;
            }

            yield return SelectContainingAtomTab("Plugins");

            float timeout = Time.unscaledTime + 1;
            while(Time.unscaledTime < timeout)
            {
                yield return null;

                if(!script.UITransform)
                {
                    continue;
                }

                /* Close any currently open plugin UI before opening this plugin's UI */
                foreach(Transform scriptController in script.manager.pluginContainer)
                {
                    var mvrScript = scriptController.gameObject.GetComponent<MVRScript>();
                    if(mvrScript && mvrScript != script)
                    {
                        mvrScript.UITransform.gameObject.SetActive(false);
                    }
                }

                if(script.enabled)
                {
                    script.UITransform.gameObject.SetActive(true);
                    postAction?.Invoke();
                    yield break;
                }
            }
        }

        protected IEnumerator SelectContainingAtomTab(string tabName, Action postAction = null)
        {
            if(SuperController.singleton.gameMode != SuperController.GameMode.Edit)
            {
                SuperController.singleton.gameMode = SuperController.GameMode.Edit;
            }

            SuperController.singleton.SelectController(script.containingAtom.mainController, false, false);
            SuperController.singleton.ShowMainHUDAuto();

            float timeout = Time.unscaledTime + 1;
            while(Time.unscaledTime < timeout)
            {
                yield return null;
                var selector = script.containingAtom.gameObject.GetComponentInChildren<UITabSelector>();
                if(selector)
                {
                    selector.SetActiveTab(tabName);
                    postAction?.Invoke();
                    break;
                }
            }
        }
    }
}
