using UnityEngine;

namespace Naturalis.Common.Components
{
    sealed class FreeControllerState
    {
        readonly FreeControllerV3 _controller;
        readonly FreeControllerV3.PositionState _positionState;
        readonly FreeControllerV3.RotationState _rotationState;

        public FreeControllerState(FreeControllerV3 controller)
        {
            _controller = controller;
            _positionState = _controller.currentPositionState;
            _rotationState = _controller.currentRotationState;
        }

        public void SetStateOn()
        {
            _controller.currentPositionState = FreeControllerV3.PositionState.On;
            _controller.currentRotationState = FreeControllerV3.RotationState.On;
        }

        public void Restore()
        {
            _controller.currentPositionState = _positionState;
            _controller.currentRotationState = _rotationState;
        }

        public static bool ControllerIsLinked(FreeControllerV3 controller)
        {
            return controller.currentPositionState == FreeControllerV3.PositionState.ParentLink ||
                controller.currentPositionState == FreeControllerV3.PositionState.PhysicsLink ||
                controller.currentRotationState == FreeControllerV3.RotationState.ParentLink ||
                controller.currentRotationState == FreeControllerV3.RotationState.PhysicsLink;
        }
    }
}
