﻿using UnityEngine;

namespace Naturalis.Common.Components
{
    public sealed class StorableFloat : JSONStorableFloat, IStorable
    {
        const float TOLERANCE = 0.00001f;
        float _calibratedValue;

        public string Name()
        {
            return name;
        }

        public bool IsModified()
        {
            return Mathf.Abs(val - defaultVal) > TOLERANCE;
        }

        public bool IsCalibrated()
        {
            return Mathf.Abs(val - _calibratedValue) < TOLERANCE;
        }

        public StorableFloat(
            string paramName,
            float startingValue,
            float minValue,
            float maxValue,
            bool constrain = true,
            bool interactable = true
        ) : base(paramName, startingValue, minValue, maxValue, constrain, interactable)
        {
        }

        public void SetValueAndDefault(float value)
        {
            defaultVal = value;
            val = value;
        }

        public void SaveCalibrated()
        {
            _calibratedValue = val;
        }
    }
}
