﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace Naturalis.Common.Components
{
    class UIModManagerBase : MonoBehaviour
    {
        protected Transform parentUI;
        protected readonly List<UIMod> uiMods = new List<UIMod>();
        protected static bool initialized;
        protected JSONStorableBool skinMaterials2Modified;

        protected UIMod NewUIMod(
            string uiModId,
            string targetName,
            Func<UIMod, IEnumerator> changesFunc,
            Func<bool> checkPluginEnabled,
            bool useDisabledStyle = false
        )
        {
            var uiMod = gameObject.AddComponent<UIMod>();
            uiMod.Init(uiModId, parentUI, targetName, changesFunc);
            uiMod.checkPluginEnabled = checkPluginEnabled;
            uiMod.useDisabledStyle = useDisabledStyle;
            uiMods.Add(uiMod);
            return uiMod;
        }

        protected void ShowFrictionUI(bool frictionHandlerEnabled)
        {
            if(!enabled)
            {
                return;
            }

            if(frictionHandlerEnabled)
            {
                uiMods.Find(uiMod => uiMod.id == Constant.SKIN_MATERIALS_2_FRICTION).Enable();
            }

            uiMods.Find(uiMod => uiMod.id == Constant.SKIN_MATERIALS_2_ADD_BTN).Disable();

            skinMaterials2Modified.val = true;
        }

        protected void HideFrictionUI(bool frictionHandlerEnabled)
        {
            if(!enabled)
            {
                return;
            }

            uiMods.Find(uiMod => uiMod.id == Constant.SKIN_MATERIALS_2_FRICTION).Disable();
            if(frictionHandlerEnabled)
            {
                uiMods.Find(uiMod => uiMod.id == Constant.SKIN_MATERIALS_2_ADD_BTN).Enable();
            }

            skinMaterials2Modified.val = false;
        }

        protected void OnDisable()
        {
            foreach(var uiMod in uiMods)
            {
                uiMod.Disable();
            }
        }

        void OnDestroy()
        {
            foreach(var uiMod in uiMods)
            {
                Destroy(uiMod);
            }
        }
    }
}
