using Naturalis.Common.Components;

namespace Naturalis.Common.Handlers.Configs
{
    sealed class PhysicsCalculator
    {
        public StorableFloat multiplierJsf { get; set; }
        public MultiplierConfig primaryConfig { get; set; }
        public MultiplierConfig secondaryConfig { get; set; }

        public float Calculate(float mass, float softness, bool usePrimary)
        {
            return Calculate(usePrimary ? primaryConfig : secondaryConfig, mass, softness);
        }

        static float Calculate(MultiplierConfig config, float valueA, float valueB)
        {
            if(config == null || config.baseValue == 0)
            {
                return 0;
            }

            float baseValue = config.baseValue;
            return baseValue +
                baseValue * config.multiplierA * config.curveA(valueA) +
                baseValue * config.multiplierB * config.curveB(valueB);
        }

        public float CalculateDynamic(float effect, float valueA, float valueB, bool usePrimary)
        {
            return CalculateDynamic(usePrimary ? primaryConfig : secondaryConfig, effect, valueA, valueB);
        }

        float CalculateDynamic(MultiplierConfig config, float effect, float valueA, float valueB)
        {
            float multiplier = (multiplierJsf?.val ?? 1) * effect;
            return multiplier *
            (
                config.baseValue +
                config.multiplierA * config.curveA(valueA) +
                config.multiplierB * config.curveB(valueB)
            );
        }
    }
}
