//
using System.Text.RegularExpressions;

namespace Naturalis.Common.Models
{
    sealed class ConfigurableMorph
    {
        readonly DAZMorph _morph;
        public JSONStorableBool isNegativeJsb { get; }
        public JSONStorableFloat softMultiplierJsf { get; } // TODO refactor as a base multiplier
        public JSONStorableFloat massMultiplierJsf { get; }
        public JSONStorableString displayValueJss { get; }
        bool _isNegative;
        float _softMultiplier;
        float _massMultiplier;

        public string morphName { get; }
        readonly string _label;
        public string morphDisplayName { get; }

        public ConfigurableMorph(DAZMorph morph, bool isNegative, JSONStorableFloat softMultiplierJsf, JSONStorableFloat massMultiplierJsf)
        {
            _morph = morph;
            morphName = _morph.morphName;
            _label = IsCenterMorph() || IsLeftRightMorph()
                ? _morph.displayName
                : Regex.Replace(_morph.displayName, @" L$", "");
            morphDisplayName = _label.Substring(morphName.IndexOf(" ") + 1);

            isNegativeJsb = new JSONStorableBool("isNegative", isNegative);
            this.softMultiplierJsf = softMultiplierJsf;
            this.massMultiplierJsf = massMultiplierJsf;
            displayValueJss = new JSONStorableString("displayValue", "");

            isNegativeJsb.setCallbackFunction = val => _isNegative = val;
            softMultiplierJsf.setCallbackFunction = val => _softMultiplier = val;
            massMultiplierJsf.setCallbackFunction = val => _massMultiplier = val;

            _isNegative = isNegativeJsb.val;
            _softMultiplier = softMultiplierJsf.val;
            _massMultiplier = massMultiplierJsf.val;
        }

        public void Update(float effect, float mass, float softness)
        {
            float value =
                softness * _softMultiplier * effect +
                mass * _massMultiplier * effect;

            bool inRange = _isNegative ? value < 0 : value > 0;
            SetValue(inRange ? Calc.RoundToDecimals(value) : 0);
        }

        public void Reset()
        {
            SetValue(0);
        }

        void SetValue(float value)
        {
            _morph.morphValue = value;
            #if ENV_DEVELOPMENT
            {
                displayValueJss.val = "\n".Size(16) + $"{morphDisplayName}: {_morph.morphValue}";
            }
            #endif
        }

        bool IsCenterMorph()
        {
            return !_morph.displayName.EndsWith(" L") && !_morph.displayName.EndsWith(" R");
        }

        bool IsLeftRightMorph()
        {
            return _morph.displayName.StartsWith("LT") || _morph.displayName.StartsWith("RT");
        }

        /* TODO broken */
        public string ToCodeString(string getMorphMethodName, string subDir = "")
        {
            string dollar = IsCenterMorph() || IsLeftRightMorph() ? "" : "$";
            string sideStr = IsCenterMorph() || IsLeftRightMorph() ? "" : " {side}";

            return $@"
            new {nameof(ConfigurableMorph)}(
                {nameof(Person)}.{getMorphMethodName}({dollar}""{subDir}{_label}{sideStr}""),
                {_isNegative.ToString().ToLower()},
                new {nameof(JSONStorableFloat)}(""softMultiplier"", {Calc.RoundToDecimals(softMultiplierJsf.val, 100):0.00}f, -3.00f, 3.00f),
                new {nameof(JSONStorableFloat)}(""massMultiplier"", {Calc.RoundToDecimals(massMultiplierJsf.val, 100):0.00}f, -3.00f, 3.00f)
            ),";
        }
    }
}
