﻿using System.Collections.Generic;
using System.Linq;
using Naturalis.Common.Handlers.Configs;

namespace Naturalis.Common.Models
{
    class PhysicsParameterGroup
    {
        public string name { get; set; }
        public string infoText { get; set; }

        public bool dependsOnPhysicsRate { get; set; }
        public bool requiresCalibration { get; set; }
        public bool rightInverted { get; set; }

        // TODO
        public bool allowOffsetOnlyLeftParam { get; set; }
        public JSONStorableBool applyChangesOnlyToLeftParamJsb { get; }

        public string gravityInfoText { get; set; }

        public int roundFactor { get; private set; }
        string _valueFormat;

        public string valueFormat
        {
            get { return _valueFormat; }
            set
            {
                roundFactor = Utils.ToRoundFactor(value);
                _valueFormat = value;
            }
        }

        public PhysicsParameter left { get; set; }
        public PhysicsParameter right { get; set; }

        protected PhysicsParameterGroup()
        {
            allowOffsetOnlyLeftParam = true;
            applyChangesOnlyToLeftParamJsb = new JSONStorableBool("applyChangesOnlyToLeftParam", false);
        }

        public virtual void SetCallbackFunctions()
        {
        }

        // ReSharper disable once UnusedMemberInSuper.Global
        public virtual void SetGravityMultiplierCallbacks()
        {
        }

        public void UpdateMinMax(float min, float max)
        {
            left.UpdateMinMax(min, max);
            right.UpdateMinMax(min, max);
        }

        public void UpdateMassValue(float mass)
        {
            left.UpdateMassValue(mass);
            right.UpdateMassValue(mass);
        }

        public void UpdateBaseValue(float mass, float softness, float quickness, bool usePrimary)
        {
            float physicsRateMultiplier = dependsOnPhysicsRate ? Utils.PhysicsRateMultiplier() : 1;
            left.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
            right.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
        }

        public void UpdateOffsetMinMax()
        {
            left.UpdateOffsetMinMax();
            right.UpdateOffsetMinMax();
        }

        public void OverrideValue(float value)
        {
            left.OverrideValue(value);
            right.OverrideValue(value);
        }

        public void SetGravityCalculators(Dictionary<string, Dictionary<string, PhysicsCalculator>> configs)
        {
            left.gravityCalculators = configs[Side.LEFT];
            right.gravityCalculators = configs[Side.RIGHT];
        }

        public void SetForceCalculators(Dictionary<string, Dictionary<string, PhysicsCalculator>> configs)
        {
            left.forceCalculators = configs[Side.LEFT];
            right.forceCalculators = configs[Side.RIGHT];
        }

        public bool IsModified()
        {
            return left.offsetJsf.val != 0 || GravityPhysicsIsModified();
        }

        protected virtual bool GravityPhysicsIsModified()
        {
            return left.HasAdjustableGravityPhysics() &&
                left.gravityCalculators.Any(kvp => kvp.Value.multiplierJsf.IsModified());
        }

        public void SaveCalibrated()
        {
            if(!requiresCalibration)
            {
                return;
            }

            left.SaveCalibrated();
            right.SaveCalibrated();
        }
    }
}
