﻿using Naturalis.Common.Handlers.Configs;

namespace Naturalis.Common.Models
{
    class SoftPhysicsParameterGroup
    {
        public string name { get; set; }
        public string infoText { get; set; }

        public bool dependsOnPhysicsRate { get; set; }
        public bool hasSoftColliderVisualization { get; set; }
        public JSONStorableBool applyChangesOnlyToLeftParamJsb { get; }

        public int roundFactor { get; private set; }
        string _valueFormat;

        public string valueFormat
        {
            get { return _valueFormat; }
            set
            {
                roundFactor = Utils.ToRoundFactor(value);
                _valueFormat = value;
            }
        }

        public float visualizationLimit { get; set; }
        public float distortionLimit { get; set; }
        public bool visualizeRelativeDistortion { get; set; }

        public SoftPhysicsParameter left { get; set; }
        public SoftPhysicsParameter right { get; set; }

        protected SoftPhysicsParameterGroup()
        {
            applyChangesOnlyToLeftParamJsb = new JSONStorableBool("applyChangesOnlyToLeftParam", false);
        }

        public virtual void SetCallbackFunctions()
        {
        }

        protected void UpdateGradient(float value, JSONStorableFloat rightJsf)
        {
            rightJsf.val = applyChangesOnlyToLeftParamJsb.val ? rightJsf.defaultVal : value;
            left.Sync();
            right.Sync();
        }

        protected void SetCompressionMaxAndMultiplier(float value)
        {
            left.ldpConfig.SetCompressionMaxAndMultiplier(value);
            right.ldpConfig.SetCompressionMaxAndMultiplier(
                applyChangesOnlyToLeftParamJsb.val ? right.ldpConfig.compressionEffectJsf.defaultVal : value
            );
        }

        protected void SetCompressionSlope(float value)
        {
            left.ldpConfig.SetCompressionSlope(value);
            right.ldpConfig.SetCompressionSlope(
                applyChangesOnlyToLeftParamJsb.val ? right.ldpConfig.compressionSensitivityJsf.defaultVal : value
            );
        }

        protected void SetStretchingMaxAndMultiplier(float value)
        {
            left.ldpConfig.SetStretchingMaxAndMultiplier(value);
            right.ldpConfig.SetStretchingMaxAndMultiplier(
                applyChangesOnlyToLeftParamJsb.val ? right.ldpConfig.stretchingEffectJsf.defaultVal : value
            );
        }

        protected void SetStretchingSlope(float value)
        {
            left.ldpConfig.SetStretchingSlope(value);
            right.ldpConfig.SetStretchingSlope(
                applyChangesOnlyToLeftParamJsb.val ? right.ldpConfig.stretchingSensitivityJsf.defaultVal : value
            );
        }

        public void UpdateBaseValue(float mass, float softness, float quickness, bool usePrimary)
        {
            float physicsRateMultiplier = dependsOnPhysicsRate ? Utils.PhysicsRateMultiplier() : 1;
            left.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
            right.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
        }

        public void UpdateOffsetMinMax()
        {
            left.UpdateOffsetMinMax();
            right.UpdateOffsetMinMax();
        }

        public void SetFrictionCalculators(PhysicsCalculator leftCalculator, PhysicsCalculator rightCalculator)
        {
            left.inverseFrictionCalculator = leftCalculator;
            right.inverseFrictionCalculator = rightCalculator;
        }

        public void UpdateInverseFrictionValue(float friction, float mass, float softness)
        {
            left.UpdateInverseFrictionValue(friction, mass, softness);
            right.UpdateInverseFrictionValue(friction, mass, softness);
        }

        public void ResetInverseFrictionValue()
        {
            left.ResetInverseFrictionValue();
            right.ResetInverseFrictionValue();
        }

        public float CalculateDistortionOffset(string side, float distortionBase, float distortion)
        {
            return side == Side.LEFT
                ? left.ldpConfig.CalculateDistortionOffset(distortionLimit, distortionBase, distortion)
                : right.ldpConfig.CalculateDistortionOffset(distortionLimit, distortionBase, distortion);
        }

        public bool IsModified()
        {
            return left.offsetJsf.val != 0 ||
                left.gradientConfig.IsModified() ||
                left.ldpConfig.IsModified();
        }
    }
}
