using UnityEngine;
using UnityEngine.Events;

namespace Naturalis.Common.UI
{
    sealed class NavigationButton
    {
        readonly UIDynamicButton _uiDynamicButton;
        bool _disabled;
        readonly Color _activeColor;

        public NavigationButton(UIDynamicButton uiDynamicButton, string label, Color activeColor)
        {
            _uiDynamicButton = uiDynamicButton;
            _uiDynamicButton.label = label;
            _activeColor = activeColor;
        }

        public void SetActive()
        {
            if(_disabled)
            {
                return;
            }

            _uiDynamicButton.textColor = _activeColor;
            var colors = _uiDynamicButton.button.colors;
            colors.highlightedColor = Color.white;
            colors.pressedColor = Color.white;
            _uiDynamicButton.button.colors = colors;
        }

        public void SetInactive()
        {
            if(_disabled)
            {
                return;
            }

            _uiDynamicButton.textColor = Color.white;
            _uiDynamicButton.buttonColor = Colors.darkerGray;
            var colors = _uiDynamicButton.button.colors;
            colors.highlightedColor = Colors.defaultBtnHighlightedColor;
            colors.pressedColor = Colors.defaultBtnPressedColor;
            _uiDynamicButton.button.colors = colors;
        }

        public void SetDisabled()
        {
            _disabled = true;
            _uiDynamicButton.textColor = new Color(0.8f, 0.8f, 0.8f);
            _uiDynamicButton.buttonColor = Colors.darkerGray;
            _uiDynamicButton.button.interactable = false;
        }

        public void AddListener(UnityAction unityAction)
        {
            _uiDynamicButton.AddListener(unityAction);
        }
    }
}
