﻿using System;
using MVR.FileManagementSecure;
using SimpleJSON;
using System.Collections.Generic;

namespace Naturalis.Common
{
    static class FileUtils
    {
        const string DATA_DIR = "Custom/PluginData/everlaster/Naturalis";
        public const string SAVES_DIR = "Saves/PluginData/Naturalis";

        public static JSONClass ReadDataJSON(string subPath)
        {
            EnsureDirExists(DATA_DIR);
            string path = $"{DATA_DIR}/{subPath}";
            JSONClass jc = null;

            try
            {
                if(FileManagerSecure.FileExists(path))
                {
                    jc = JSON.Parse(FileManagerSecure.ReadAllText(path)).AsObject;
                }
            }
            catch(Exception e)
            {
                Loggr.Error($"{nameof(FileUtils)}.{nameof(ReadDataJSON)}: {e}");
            }

            return jc;
        }

        public static void WriteDataJSON(string subPath, JSONClass jc, UserActionCallback confirmCallback = null)
        {
            EnsureDirExists(DATA_DIR);
            string path = $"{DATA_DIR}/{subPath}";
            FileManagerSecure.WriteAllText(
                path,
                jc.ToString(string.Empty),
                confirmCallback,
                () => Loggr.Message("Save to file cancelled."),
                err => Loggr.Error($"Save to file error: {err}")
            );
        }

        public static void OpenDataDirInExplorer()
        {
            EnsureDirExists(DATA_DIR);
            SuperController.singleton.OpenFolderInExplorer(DATA_DIR);
        }

        public static List<ShortCut> GetShortCutsForDirectory(string path)
        {
            var cutsForDirectory = FileManagerSecure.GetShortCutsForDirectory(path, true, true, true);
            var rootShortCut = new ShortCut
            {
                displayName = "Root",
                path = FileManagerSecure.GetFullPath("."),
            };
            cutsForDirectory.Insert(0, rootShortCut);
            return cutsForDirectory;
        }

        public static void EnsureDirExists(string dirName)
        {
            if(!DirectoryExists(dirName))
            {
                FileManagerSecure.CreateDirectory(dirName);
            }
        }

        public static bool DirectoryExists(string dirName)
        {
            return FileManagerSecure.DirectoryExists(dirName);
        }
    }
}
