using System;
using UnityEngine;

namespace Naturalis.Common
{
    sealed class FrequencyRunner
    {
        readonly Action _action;

        public FrequencyRunner(JSONStorableFloat frequencyJsf, Action action = null)
        {
            _action = action;
            frequencyJsf.setCallbackFunction = CalculateFrequency;
            CalculateFrequency(frequencyJsf.val);
        }

        public FrequencyRunner(JSONStorableStringChooser frequencyJssc, Action action = null)
        {
            _action = action;
            frequencyJssc.setCallbackFunction += option => CalculateFrequency(StringOptionToHz(option));
            CalculateFrequency(StringOptionToHz(frequencyJssc.val));
        }

        float _frequency;
        float _timeSinceLastCheck;

        public T Run<T>(Func<T> func)
        {
            _timeSinceLastCheck += Time.unscaledDeltaTime;
            if(_timeSinceLastCheck >= _frequency)
            {
                _timeSinceLastCheck = 0;
                return func();
            }

            return default(T);
        }

        public void Run(Action action)
        {
            _timeSinceLastCheck += Time.unscaledDeltaTime;
            if(_timeSinceLastCheck >= _frequency)
            {
                _timeSinceLastCheck = 0;
                action();
            }
        }

        public void Run()
        {
            Run(_action);
        }

        void CalculateFrequency(float value)
        {
            _frequency = 1 / (float) Mathf.RoundToInt(value);
        }

        static float StringOptionToHz(string option)
        {
            switch(option)
            {
                case "10 Hz": return 10;
                case "20 Hz": return 20;
                case "30 Hz": return 30;
                case "45 Hz": return 45;
                case "60 Hz": return 60;
                case "Use Physics Rate": return Mathf.RoundToInt(1 / Time.fixedDeltaTime);
                default: throw new ArgumentException($"Invalid option {option}");
            }
        }
    }
}
