﻿using SimpleJSON;
using UnityEngine;

namespace Naturalis.Common
{
    public static class JSONUtils
    {
        public static void RenameKey(JSONClass jc, string from, string to)
        {
            if(jc.HasKey(from))
            {
                jc[to] = jc[from];
                jc.Remove(from);
            }
        }

        public static JSONClass SerializeQuaternion(Quaternion quaternion)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = quaternion.x },
                ["y"] = { AsFloat = quaternion.y },
                ["z"] = { AsFloat = quaternion.z },
                ["w"] = { AsFloat = quaternion.w },
            };
            return jc;
        }

        public static Quaternion DeserializeQuaternion(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Quaternion(
                jc["x"].AsFloat,
                jc["y"].AsFloat,
                jc["z"].AsFloat,
                jc["w"].AsFloat
            );
        }

        public static JSONClass SerializeVector3(Vector3 vector)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = vector.x },
                ["y"] = { AsFloat = vector.y },
                ["z"] = { AsFloat = vector.z },
            };
            return jc;
        }

        public static Vector3 DeserializeVector3(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Vector3(
                jc["x"].AsFloat,
                jc["y"].AsFloat,
                jc["z"].AsFloat
            );
        }

        public static JSONClass SerializeVector2(Vector2 vector)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = vector.x },
                ["y"] = { AsFloat = vector.y },
            };
            return jc;
        }

        public static Vector2 DeserializeVector2(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Vector3(
                jc["x"].AsFloat,
                jc["y"].AsFloat
            );
        }
    }
}
