﻿// ReSharper disable MemberCanBePrivate.Global UnusedMember.Global
using System;

namespace Naturalis.Common
{
    sealed class SemVer
    {
        Version version { get; }

        public SemVer(string versionStr, bool removePrereleaseTag = false)
        {
            try
            {
                /* Without removePrereleaseTag set to true, a versionStr containing a prerelease tag
                 * will result in a null version, and all comparisons will return false.
                 */
                version = removePrereleaseTag
                    ? new Version(versionStr.Split('-')[0])
                    : new Version(versionStr);
            }
            catch(Exception)
            {
                // Debug.Log("Invalid version string: " + version);
            }
        }

        public bool IsGte(string other)
        {
            return IsGte(new SemVer(other));
        }

        public bool IsMajorGte(string other)
        {
            return IsMajorGte(new SemVer(other));
        }

        public bool IsMajorMinorGte(string other)
        {
            return IsMajorMinorGte(new SemVer(other));
        }

        public bool IsMajorMinorEqual(string other)
        {
            return IsMajorMinorEqual(new SemVer(other));
        }

        public bool IsGte(SemVer other)
        {
            if(version == null || other.version == null)
            {
                return false;
            }

            return version >= other.version;
        }

        public bool IsMajorGte(SemVer other)
        {
            if(version == null || other.version == null)
            {
                return false;
            }

            return version.Major >= other.version.Major;
        }

        public bool IsMajorMinorGte(SemVer other)
        {
            if(version == null || other.version == null)
            {
                return false;
            }

            var majorMinorA = new Version(version.Major, version.Minor);
            var majorMinorB = new Version(other.version.Major, other.version.Minor);

            return majorMinorA >= majorMinorB;
        }

        public bool IsMajorMinorEqual(SemVer other)
        {
            if(version == null || other.version == null)
            {
                return false;
            }

            var majorMinorA = new Version(version.Major, version.Minor);
            var majorMinorB = new Version(other.version.Major, other.version.Minor);

            return majorMinorA == majorMinorB;
        }
    }
}
