﻿using System;
using System.Collections;
using Naturalis.Common;
using Naturalis.Common.Components;
using UnityEngine;

namespace Naturalis.SET
{
    sealed class UIModManager : UIModManagerBase
    {
        UIMod _femaleMorphsMod;
        UIMod _maleMorphsMod;
        UIMod _skinMaterials2Mod;

        static bool CheckPluginEnabled()
        {
            return Settings.tittyMagic.IsEnabledNullSafe() || Settings.bootyMagic.IsEnabledNullSafe();
        }

        public bool MorphsTabIsVisible()
        {
            return _femaleMorphsMod && _femaleMorphsMod.IsVisible() || _maleMorphsMod && _maleMorphsMod.IsVisible();
        }

        public void ModifyAtomUI()
        {
            parentUI = Person.instance.atomUIContent;
            _femaleMorphsMod = NewUIMod(
                Constant.FEMALE_MORPHS,
                "Female Morphs",
                CreateFemaleMorphsPauseToggleCo,
                CheckPluginEnabled,
                true
            );
            _maleMorphsMod = NewUIMod(
                Constant.MALE_MORPHS,
                "Male Morphs",
                CreateMaleMorphsPauseToggleCo,
                CheckPluginEnabled,
                true
            );
            _skinMaterials2Mod = NewUIMod(
                Constant.SKIN_MATERIALS_2_MOVE_BTN_GROUP,
                "Skin Materials 2",
                MoveSkinMaterialsUIButtonGroupCo,
                CheckPluginEnabled
            );

            foreach(var uiMod in uiMods)
            {
                uiMod.Apply();
            }

            if(!Settings.bootyMagic.IsEnabledNullSafe() && !Settings.tittyMagic.IsEnabledNullSafe())
            {
                enabled = false;
            }
        }

        static IEnumerator CreateFemaleMorphsPauseToggleCo(UIMod uiMod)
        {
            while(!Settings.IsInitialized())
            {
                yield return null;
            }

            try
            {
                CreatePauseToggle(uiMod, Settings.instance.pauseFemaleMorphUpdatesJsb);
                uiMod.targetUIEventsListener = uiMod.target.gameObject.AddComponent<UnityEventsListener>();
            }
            catch(Exception e)
            {
                Loggr.Error($"Error modifying {uiMod.id} UI: {e}");
            }
        }

        static IEnumerator CreateMaleMorphsPauseToggleCo(UIMod uiMod)
        {
            while(!Settings.IsInitialized())
            {
                yield return null;
            }

            try
            {
                CreatePauseToggle(uiMod, Settings.instance.pauseMaleMorphUpdatesJsb);
                uiMod.targetUIEventsListener = uiMod.target.gameObject.AddComponent<UnityEventsListener>();
            }
            catch(Exception e)
            {
                Loggr.Error($"Error modifying {uiMod.id} UI: {e}");
            }
        }

        static void CreatePauseToggle(UIMod uiMod, JSONStorableBool storable)
        {
            var rightSide = uiMod.target.Find("LeftSide");
            var customTransform = Utils.DestroyLayout(Settings.initScript.InstantiateToggle(rightSide));
            var rectTransform = customTransform.GetComponent<RectTransform>();
            rectTransform.pivot = new Vector2(0, 0);
            rectTransform.anchoredPosition = new Vector2(0, 249);
            rectTransform.sizeDelta = new Vector2(-10, 50);
            uiMod.AddCustomObject(customTransform.gameObject);
            var uiDynamicToggle = customTransform.GetComponent<UIDynamicToggle>();
            Settings.initScript.AddToggleToJsb(uiDynamicToggle, storable);
            uiDynamicToggle.label = $"{nameof(Naturalis)}: Pause morph updates";
        }

        static IEnumerator MoveSkinMaterialsUIButtonGroupCo(UIMod uiMod)
        {
            if(!Person.IsFemale())
            {
                yield break;
            }

            while(!Settings.IsInitialized())
            {
                yield return null;
            }

            var rightSide = uiMod.target.Find("RightSide");
            uiMod.RepositionTransform(rightSide.Find("SavePanel"), new Vector2(-440, 70), new Vector2(-440, 670));
            uiMod.RepositionTransform(rightSide.Find("RestorePanel"), new Vector2(-230, 70), new Vector2(-230, 670));
            uiMod.RepositionTransform(rightSide.Find("Reset"), new Vector2(-70, 70), new Vector2(-70, 670));

            if(Person.HasDummySkin() || !(Settings.tittyMagic.IsEnabledNullSafe() || Settings.bootyMagic.IsEnabledNullSafe()))
            {
                uiMod.Disable();
            }
        }

        public void SetSkinMaterials2ModEnabled(bool value)
        {
            if(value)
            {
                if(!Person.HasDummySkin())
                {
                    _skinMaterials2Mod.Enable();
                }
            }
            else
            {
                _skinMaterials2Mod.Disable();
            }
        }

        bool _enableSkinMaterials2Mod;

        public void SetEnabled(bool enableSkinMaterials2Mod = false)
        {
            _enableSkinMaterials2Mod = enableSkinMaterials2Mod;
            enabled = true;
        }

        void OnEnable()
        {
            foreach(var uiMod in uiMods)
            {
                if(_enableSkinMaterials2Mod && uiMod.id == Constant.SKIN_MATERIALS_2_MOVE_BTN_GROUP)
                {
                    if(!Person.HasDummySkin())
                    {
                        uiMod.Enable();
                    }
                }
                else
                {
                    uiMod.Enable();
                }
            }
        }
    }
}
