﻿using Naturalis.Common;
using Naturalis.Common.Handlers.Configs;
using Naturalis.Common.Models;

namespace Naturalis.TM.Handlers
{
    static class NippleErectionHandler
    {
        static BasicMorph _morph;
        public static JSONStorableFloat nippleErectionJsf { get; private set; }
        static PhysicsCalculator _springCalculator;
        static PhysicsCalculator _damperCalculator;

        public static void Init()
        {
            nippleErectionJsf = TittyMagic.script.NewStorableFloat("Nipple Erection", 0.00f, 0.00f, 1.00f);
            nippleErectionJsf.setCallbackFunction = _ => Update();
            LoadSettings();
        }

        static void LoadSettings()
        {
            _morph = new BasicMorph(Person.GetBreastMorph("TM_NippleErection"), 1.0f);
            if(Person.IsFemale())
            {
                _springCalculator = new PhysicsCalculator
                {
                    primaryConfig = new MultiplierConfig
                    {
                        baseValue = 0.50f,
                        multiplierA = 0.75f,
                        multiplierB = 0,
                        curveA = x => 1 - x,
                    },
                };
                _damperCalculator = new PhysicsCalculator
                {
                    primaryConfig = new MultiplierConfig
                    {
                        baseValue = 0.50f,
                        multiplierA = 0.75f,
                        multiplierB = 0,
                        curveA = x => 1 - x,
                    },
                };
            }
        }

        public static void Update()
        {
            _morph.Update(nippleErectionJsf.val);
            if(SoftPhysicsHandler.softPhysicsOn)
            {
                float mass = MainPhysicsHandler.normalizedMass;
                float softness = TittyMagic.softness;
                float springMultiplier = 1 + _springCalculator.CalculateDynamic(nippleErectionJsf.val, mass, softness, true);
                float damperMultiplier = 1 + _damperCalculator.CalculateDynamic(nippleErectionJsf.val, mass, softness, true);
                SoftPhysicsHandler.nippleSoftJoints[Side.LEFT].SetAdditionalBaseMultiplier(FAT_SPRING.NAME, springMultiplier);
                SoftPhysicsHandler.nippleSoftJoints[Side.LEFT].SetAdditionalBaseMultiplier(FAT_DAMPER.NAME, damperMultiplier);
                SoftPhysicsHandler.nippleSoftJoints[Side.RIGHT].SetAdditionalBaseMultiplier(FAT_SPRING.NAME, springMultiplier);
                SoftPhysicsHandler.nippleSoftJoints[Side.RIGHT].SetAdditionalBaseMultiplier(FAT_DAMPER.NAME, damperMultiplier);
            }
        }

        public static void Reset()
        {
            _morph.Reset();
            // physics values are not reset here but by SoftPhysicsHandler
        }

        public static void Destroy()
        {
            _morph = null;
            nippleErectionJsf = null;
            _springCalculator = null;
            _damperCalculator = null;
        }
    }
}
