using Naturalis.Common.Components;
using Naturalis.Common.Models;
using Naturalis.TM.Handlers;

namespace Naturalis.TM.Models
{
    sealed class BreastPhysicsParameterGroup : PhysicsParameterGroup
    {
        public override void SetCallbackFunctions()
        {
            left.offsetJsf.setJSONCallbackFunction = jsf =>
            {
                var leftStorable = (StorableFloat) jsf;
                var rightStorable = right.offsetJsf;
                left.UpdateOffsetValue(leftStorable.val);
                float rightValue = rightInverted ? -leftStorable.val : leftStorable.val;
                right.UpdateOffsetValue(applyChangesOnlyToLeftParamJsb.val ? rightStorable.defaultVal : rightValue);
                if(requiresCalibration)
                {
                    TittyMagic.script.SetCalibrationPending(leftStorable);
                }

                TittyMagic.tabs.physicsWindow.OnMainParamAdjusted(this);
            };
            applyChangesOnlyToLeftParamJsb.setCallbackFunction = value =>
            {
                /* Offset */
                {
                    var leftStorable = left.offsetJsf;
                    var rightStorable = right.offsetJsf;
                    float rightValue = rightInverted ? -leftStorable.val : leftStorable.val;
                    right.UpdateOffsetValue(value ? rightStorable.defaultVal : rightValue);
                    if(requiresCalibration)
                    {
                        if(name == BREAST_JOINT_MASS.NAME)
                        {
                            TittyMagic.script.calibrationHelper.updateMassWhenCalibrating = true;
                        }

                        TittyMagic.script.SetCalibrationPending(rightStorable);
                    }
                }

                /* Gravity */
                {
                    foreach(var kvp in left.gravityCalculators)
                    {
                        var config = kvp.Value;
                        var leftStorable = config.multiplierJsf;
                        var rightStorable = right.gravityCalculators[kvp.Key].multiplierJsf;
                        rightStorable.val = applyChangesOnlyToLeftParamJsb.val ? rightStorable.defaultVal : leftStorable.val;
                        if(requiresCalibration)
                        {
                            TittyMagic.script.SetCalibrationPending(rightStorable);
                        }
                    }
                }
            };
        }

        public override void SetGravityMultiplierCallbacks()
        {
            foreach(var kvp in left.gravityCalculators)
            {
                var config = kvp.Value;
                config.multiplierJsf.setJSONCallbackFunction = jsf =>
                {
                    var leftStorable = (StorableFloat) jsf;
                    var rightStorable = right.gravityCalculators[kvp.Key].multiplierJsf;
                    rightStorable.val = applyChangesOnlyToLeftParamJsb.val ? rightStorable.defaultVal : leftStorable.val;
                    if(requiresCalibration)
                    {
                        TittyMagic.script.SetCalibrationPending(leftStorable);
                    }

                    TittyMagic.tabs.physicsWindow.OnMainParamAdjusted(this);
                };
            }
        }

        protected override bool GravityPhysicsIsModified()
        {
            return name == BREAST_JOINT_MASS.NAME
                ? GravityPhysicsHandler.gravityCounterforceJsf.IsModified()
                : base.GravityPhysicsIsModified();
        }
    }
}
