using System;
using System.Linq;
using Naturalis.Common;
using Naturalis.Common.Models;
using Naturalis.Common.UI;
using Naturalis.TM.Handlers;
using UnityEngine;
using UnityEngine.Events;

namespace Naturalis.TM.UI
{
    sealed class DevWindow : WindowBase
    {
        readonly JSONStorableString _leftDebugArea;
        readonly JSONStorableString _jointDiagnosticsJss;
        readonly JSONStorableStringChooser _softParamJssc;
        SoftPhysicsParameterGroup _selectedParam;

        public DevWindow(UnityAction onReturnToParent = null) : base(TittyMagic.script, nameof(DevWindow), onReturnToParent)
        {
            _leftDebugArea = new JSONStorableString("leftDebugArea", "");
            _jointDiagnosticsJss = new JSONStorableString("jointDiagnostics", "");
            var choices = SoftPhysicsHandler.softParameterGroups.Select(group => group.name).ToList();
            _softParamJssc = new JSONStorableStringChooser("softPhysicsParams", choices, choices.First(), "Params")
            {
                setCallbackFunction = OnParamOptionSelected,
            };
            OnParamOptionSelected(_softParamJssc.val);
        }

        void OnParamOptionSelected(string value)
        {
            if(Person.IsFemale())
            {
                _selectedParam = SoftPhysicsHandler.softParameterGroups.First(group => group.name == value);
            }
        }

        protected override void OnBuild()
        {
            AddSpacer(50, true);
            AddElement(() =>
            {
                var textField = script.CreateTextField(_leftDebugArea);
                textField.UItext.fontSize = 28;
                textField.height = Constant.UI_MAX_HEIGHT - 130;
                return textField;
            });

            AddElement(() =>
            {
                var popup = script.CreatePopupAuto(SoftPhysicsHandler.softJointDiagnostics.softJointRbDiagnosticJssc, true, 480f);
                popup.popup.labelText.color = Color.black;
                return popup;
            });

            AddElement(() =>
            {
                var popup = script.CreatePopupAuto(_softParamJssc, true);
                popup.popup.labelText.color = Color.black;
                return popup;
            });

            AddElement(() =>
            {
                var textField = script.CreateTextField(_jointDiagnosticsJss, true);
                textField.UItext.fontSize = 28;
                textField.height = 250;
                return textField;
            });
        }

        protected override void OnClose()
        {
            if(Person.IsFemale())
            {
                SoftPhysicsHandler.softJointDiagnostics.DrawServiceSetEnabled(false);
            }
        }

        public void Update()
        {
            // DebugSoftJoints();
        }

        void DebugSoftJoints()
        {
            try
            {
                if(!Person.IsFemale())
                {
                    return;
                }

                if(_selectedParam != null)
                {
                    if(Time.frameCount % 6 == 0)
                    {
                        _jointDiagnosticsJss.val = SoftPhysicsHandler.softJointDiagnostics.GetSelectedJointValues(_selectedParam);
                    }
                }

                SoftPhysicsHandler.softJointDiagnostics.ClearDiagnostics();
                SoftPhysicsHandler.softJointDiagnostics.DrawSelectedParam();
                if(_selectedParam != null)
                {
                    SoftPhysicsHandler.softJointDiagnostics.DrawDiagnostics(_selectedParam, _selectedParam.name);
                }
            }
            catch(Exception e)
            {
                Debug.Log($"{nameof(DebugSoftJoints)}: {e}");
            }
        }
    }
}
