using System.Linq;
using Naturalis.Common;
using Naturalis.Common.UI;
using Naturalis.TM.Handlers;
using UnityEngine;
using UnityEngine.UI;

namespace Naturalis.TM.UI
{
    sealed class MorphsWindow : WindowBase
    {
        public MorphsWindow() : base(TittyMagic.script, nameof(MorphsWindow))
        {
        }

        protected override void OnBuild()
        {
            BuildLeftSide();
            BuildRightSide();

            GetElement(ForceMorphHandler.baseJsf.name).AddListener(UpdateAllSliderColors);
            UpdateAllSliderColors(0);
        }

        void BuildLeftSide(bool rightSide = false)
        {
            AddHeaderTextField("Directional Force Morphing", rightSide);

            AddSpacer(5, rightSide);
            AddElement(ForceMorphHandler.baseJsf.name, () =>
            {
                var slider = script.CreateSlider(ForceMorphHandler.baseJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Base Multiplier";
                return slider;
            });

            AddInfoTextField(
                "Control how much breast shape is adjusted dynamically with morphs." +
                "\n\nThe amount of morphing is based on the breast's movement away from its" +
                " neutral position relative to the chest. The neutral position is estimated as the" +
                " position when the person is in an upright pose and only the force of gravity is applied." +
                "\n\nAnything that causes the breast to move will cause morphing: collision, gravity" +
                " or any kind of animation.",
                rightSide,
                620,
                28
            );

            AddHeaderTextField("Other Settings", rightSide);

            string nippleErectionText = Person.IsFemale()
                ? "Expand nipple morphs and harden nipple physics."
                : "Expand nipple morphs.";
            nippleErectionText += "\n\nCompensate for the droop caused by negative Up/Down Angle Target value.";
            AddInfoTextField(nippleErectionText, rightSide, 180, 28);
        }

        void BuildRightSide(bool rightSide = true)
        {
            AddSpacer(80, rightSide);
            AddMultiplierSlider(ForceMorphHandler.upJsf, "Up", rightSide);
            AddMultiplierSlider(ForceMorphHandler.depthOutJsf, "Depth Out", rightSide);
            AddMultiplierSlider(ForceMorphHandler.depthInJsf, "Depth In", rightSide);
            AddMultiplierSlider(ForceMorphHandler.sidewaysInJsf, "Sideways In", rightSide);
            AddMultiplierSlider(ForceMorphHandler.sidewaysOutJsf, "Sideways Out", rightSide);

            AddSpacer(90, rightSide);
            AddElement(() =>
            {
                var slider = script.CreateSlider(NippleErectionHandler.nippleErectionJsf, rightSide);
                slider.valueFormat = "F2";
                return slider;
            });

            AddElement(() =>
            {
                var slider = script.CreateSlider(GravityOffsetMorphHandler.offsetMorphingJsf, rightSide);
                slider.valueFormat = "F2";
                return slider;
            });
        }

        void AddMultiplierSlider(JSONStorableFloat storable, string label, bool rightSide)
        {
            AddElement(storable.name, () =>
            {
                var slider = script.CreateSlider(storable, rightSide);
                slider.valueFormat = "F2";
                slider.label = label;
                slider.AddListener((float _) => UpdateSliderColor(storable));
                return slider;
            });
        }

        void UpdateAllSliderColors(float _)
        {
            UpdateSliderColor(ForceMorphHandler.upJsf);
            UpdateSliderColor(ForceMorphHandler.depthOutJsf);
            UpdateSliderColor(ForceMorphHandler.depthInJsf);
            UpdateSliderColor(ForceMorphHandler.sidewaysInJsf);
            UpdateSliderColor(ForceMorphHandler.sidewaysOutJsf);
        }

        void UpdateSliderColor(JSONStorableFloat storable)
        {
            var slider = GetElementAs<UIDynamicSlider>(storable.name);
            if(slider)
            {
                var images = slider.slider.gameObject.transform.GetComponentsInChildren<Image>();
                var fillImage = images.First(image => image.name == "Fill");
                var handleImage = images.First(image => image.name == "Handle");
                var color = MultiplierSliderColor(ForceMorphHandler.baseJsf.val * storable.val);
                fillImage.color = color;
                handleImage.color = color;
            }
        }

        static Color MultiplierSliderColor(float value)
        {
            return value <= 1
                ? Color.Lerp(new Color(1, 1, 1, 0.25f), Color.white, value)
                : Color.Lerp(Color.white, new Color(1.0f, 0.2f, 0.2f), (value - 1) / 3);
        }
    }
}
