using System;
using System.Linq;
using Naturalis.Common;
using Naturalis.Common.Models;
using Naturalis.Common.UI;
using Naturalis.TM.Handlers;
using UnityEngine;
using UnityEngine.Events;

namespace Naturalis.TM.UI
{
    sealed class SoftParameterWindow : WindowBase
    {
        readonly SoftPhysicsParameterGroup _parameterGroup;
        readonly SoftPhysicsParameter _parameter;

        readonly JSONStorableString _outerValueJss;
        readonly JSONStorableString _mainValueJss;
        readonly JSONStorableString _areolaValueJss;
        readonly JSONStorableString _nippleValueJss;

        public SoftParameterWindow(SoftPhysicsParameterGroup parameterGroup, UnityAction onReturnToParent)
            : base(TittyMagic.script, parameterGroup.name, onReturnToParent)
        {
            _outerValueJss = new JSONStorableString("outerValue", "");
            _mainValueJss = new JSONStorableString("mainValue", "");
            _areolaValueJss = new JSONStorableString("areolaValue", "");
            _nippleValueJss = new JSONStorableString("nippleValue", "");
            _parameterGroup = parameterGroup;
            _parameter = _parameterGroup.left;
        }

        protected override void OnBuild()
        {
            BuildLeftSide();
            BuildRightSide();
            Refresh();
            SoftPhysicsHandler.softJointDiagnostics.DrawServiceSetEnabled(true);
            SetShowColliderPreviews();
        }

        void BuildLeftSide(bool rightSide = false)
        {
            AddInfoTextField(
                "\n".Size(8) + _parameterGroup.infoText,
                rightSide,
                225,
                28
            );

            AddElement(() =>
            {
                string text = "\n".Size(8) + "Local Distortion Physics".Bold();
                var textField = CreateHeaderTextField(text, 30, 50, rightSide);
                textField.UItext.alignment = TextAnchor.LowerCenter;
                return textField;
            });

            AddInfoTextField(
                "Effect".Bold() +
                " causes the value to increase (if > 0) or decrease (if < 0)" +
                " as joints move closer to (compress) or further from (stretch) each other." +
                " Sensitivity".Bold() +
                " controls how easily the value changes due to small distortions.",
                rightSide,
                160
            );

            AddElement(SoftPhysicsHandler.softJointDiagnostics.visualizeLdpJsb.name, () =>
            {
                var toggle = script.CreateToggle(SoftPhysicsHandler.softJointDiagnostics.visualizeLdpJsb, rightSide);
                toggle.label = "Visualize";
                return toggle;
            });

            var ldpConfig = _parameter.ldpConfig;
            AddElement(ldpConfig.compressionEffectJsf.name, () =>
            {
                var slider = script.CreateSlider(ldpConfig.compressionEffectJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Compression Effect";
                return slider;
            });

            AddElement(ldpConfig.compressionSensitivityJsf.name, () =>
            {
                var slider = script.CreateSlider(ldpConfig.compressionSensitivityJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Compression Sensitivity";
                return slider;
            });

            AddElement(ldpConfig.stretchingEffectJsf.name, () =>
            {
                var slider = script.CreateSlider(ldpConfig.stretchingEffectJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Stretching Effect";
                return slider;
            });

            AddElement(ldpConfig.stretchingSensitivityJsf.name, () =>
            {
                var slider = script.CreateSlider(ldpConfig.stretchingSensitivityJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Stretching Sensitivity";
                return slider;
            });

            if(_parameterGroup.hasSoftColliderVisualization)
            {
                AddElement(() =>
                {
                    var parent = script.UITransform;
                    var fieldTransform = Utils.DestroyLayout(script.InstantiateToggle(parent));
                    var rectTransform = fieldTransform.GetComponent<RectTransform>();
                    rectTransform.pivot = new Vector2(0, 0);
                    rectTransform.anchoredPosition = UISideAlign.globalSide == UISideAlign.Side.Left
                        ? new Vector2(1100, -1085)
                        : new Vector2(-550, -1085);
                    rectTransform.sizeDelta = new Vector2(-555, 50);
                    // script.uiModManager.SetDevUISliderCallbacks(rectTransform);
                    var toggle = fieldTransform.GetComponent<UIDynamicToggle>();
                    toggle.label = SoftPhysicsHandler.showColliderPreviewsJsb.name;
                    TittyMagic.script.AddToggleToJsb(toggle, SoftPhysicsHandler.showColliderPreviewsJsb);
                    return toggle;
                });

                AddElement(() =>
                {
                    var parent = script.UITransform;
                    var fieldTransform = Utils.DestroyLayout(script.InstantiateSlider(parent));
                    var rectTransform = fieldTransform.GetComponent<RectTransform>();
                    rectTransform.pivot = new Vector2(0, 0);
                    rectTransform.anchoredPosition = UISideAlign.globalSide == UISideAlign.Side.Left
                        ? new Vector2(1100, -1220)
                        : new Vector2(-550, -1220);
                    rectTransform.sizeDelta = new Vector2(-555, 120);
                    // script.uiModManager.SetDevUISliderCallbacks(rectTransform);
                    var slider = fieldTransform.GetComponent<UIDynamicSlider>();
                    slider.label = "Collider Previews Opacity";
                    TittyMagic.script.AddSliderToJsf(slider, SoftPhysicsHandler.colliderVisualizer.PreviewOpacityJSON);
                    return slider;
                });
            }
        }

        void BuildRightSide(bool rightSide = true)
        {
            AddElement(() =>
            {
                var toggle = script.CreateToggle(_parameterGroup.applyChangesOnlyToLeftParamJsb, rightSide);
                toggle.label = "Apply Changes Only To Left Breast";
                return toggle;
            });

            AddElement(() =>
            {
                var slider = script.CreateSlider(_parameter.offsetJsf, rightSide);
                slider.valueFormat = _parameterGroup.valueFormat;
                slider.label = "Offset";
                return slider;
            });

            AddSpacer(90, rightSide);
            AddElement(
                () => GroupValueTextField(_outerValueJss, new Vector2(550, -320))
            );

            AddElement(
                () => GroupValueTextField(_mainValueJss, new Vector2(550, -365))
            );

            AddElement(
                () => GroupValueTextField(_areolaValueJss, new Vector2(819, -320))
            );

            AddElement(
                () => GroupValueTextField(_nippleValueJss, new Vector2(819, -365))
            );

            AddElement(() =>
            {
                string text = "\n".Size(8) + "Value Gradient".Bold();
                var textField = CreateHeaderTextField(text, 30, 50, rightSide);
                textField.UItext.alignment = TextAnchor.LowerCenter;
                return textField;
            });

            AddInfoTextField(
                "Middle Shift".Bold() +
                " moves the gradient's mid point from the outer joints" +
                " at -1.00 to the areola joints at 1.00. The " +
                "Multiplier".Bold() +
                " sliders set target" +
                " values for the gradient's end points.",
                rightSide,
                160
            );

            var visualizeGradient = SoftPhysicsHandler.softJointDiagnostics.visualizeGradientJsb;
            AddElement(() =>
            {
                var toggle = script.CreateToggle(visualizeGradient, rightSide);
                toggle.label = "Visualize";
                return toggle;
            });

            var gradientConfig = _parameter.gradientConfig;
            AddElement(() =>
            {
                var slider = script.CreateSlider(gradientConfig.middleShiftJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Middle Shift";
                return slider;
            });

            AddElement(() =>
            {
                var slider = script.CreateSlider(gradientConfig.outerMultiplierJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Outer Multiplier";
                return slider;
            });

            AddElement(() =>
            {
                var slider = script.CreateSlider(gradientConfig.centerMultiplierJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Areola Multiplier";
                return slider;
            });

            AddElement(() =>
            {
                var slider = script.CreateSlider(gradientConfig.nippleMultiplierJsf, rightSide);
                slider.valueFormat = "F2";
                slider.label = "Nipple Multiplier";
                return slider;
            });

            // AddElement(() =>
            // {
            //     var slider = script.CreateSlider(_parameter.gradientConfig.outerToMiddleCurvatureJsf, rightSide);
            //     slider.valueFormat = "F2";
            //     slider.label = "Outer To Middle Curvature";
            //     return slider;
            // });

            // AddElement(() =>
            // {
            //     var slider = script.CreateSlider(_parameter.gradientConfig.outerToMiddleMidPointJsf, rightSide);
            //     slider.valueFormat = "F2";
            //     slider.label = "Outer To Middle Mid Point";
            //     return slider;
            // });

            // AddElement(() =>
            // {
            //     var slider = script.CreateSlider(_parameter.gradientConfig.middleToCenterCurvatureJsf, rightSide);
            //     slider.valueFormat = "F2";
            //     slider.label = "Middle To Center Curvature";
            //     return slider;
            // });

            // AddElement(() =>
            // {
            //     var slider = script.CreateSlider(_parameter.gradientConfig.middleToCenterMidPointJsf, rightSide);
            //     slider.valueFormat = "F2";
            //     slider.label = "Middle To Center Mid Point";
            //     return slider;
            // });
        }

        static UIDynamicTextField GroupValueTextField(JSONStorableString storable, Vector2 position)
        {
            var parent = TittyMagic.script.UITransform.Find("Scroll View/Viewport/Content");
            var fieldTransform = Utils.DestroyLayout(TittyMagic.script.InstantiateTextField(parent));
            var rectTransform = fieldTransform.GetComponent<RectTransform>();
            rectTransform.pivot = new Vector2(0, 0);
            rectTransform.anchoredPosition = position;
            rectTransform.sizeDelta = new Vector2(-825, 42);
            var textField = fieldTransform.GetComponent<UIDynamicTextField>();
            textField.text = storable.val;
            textField.UItext.fontSize = 26;
            TittyMagic.script.AddTextFieldToJss(textField, storable);
            return textField;
        }

        public void Refresh()
        {
            var ldpConfig = _parameter.ldpConfig;
            bool active = SoftPhysicsHandler.LdpUpdateIsEnabled();
            GetElement(SoftPhysicsHandler.softJointDiagnostics.visualizeLdpJsb.name)
                .SetActiveStyle(active, true);
            GetElement(ldpConfig.compressionEffectJsf.name)
                .SetActiveStyle(active, true);
            GetElement(ldpConfig.compressionSensitivityJsf.name)
                .SetActiveStyle(active, true);
            GetElement(ldpConfig.stretchingEffectJsf.name)
                .SetActiveStyle(active, true);
            GetElement(ldpConfig.stretchingSensitivityJsf.name)
                .SetActiveStyle(active, true);
        }

        public void SetShowColliderPreviews()
        {
            SoftPhysicsHandler.SetShowColliderPreviews(_parameterGroup.hasSoftColliderVisualization);
        }

        public void UpdateDiagnostics()
        {
            if(Time.frameCount % 6 == 0)
            {
                _outerValueJss.val = ValueInfoFieldString(SoftVerticesGroupName.OUTER);
                _mainValueJss.val = ValueInfoFieldString(SoftVerticesGroupName.MAIN);
                _areolaValueJss.val = ValueInfoFieldString(SoftVerticesGroupName.AREOLA);
                _nippleValueJss.val = ValueInfoFieldString(SoftVerticesGroupName.NIPPLE);
            }

            SoftPhysicsHandler.softJointDiagnostics.ClearDiagnostics();
            SoftPhysicsHandler.softJointDiagnostics.DrawDiagnostics(_parameterGroup, _parameterGroup.name);
            if(_parameterGroup.hasSoftColliderVisualization && SoftPhysicsHandler.showColliderPreviewsJsb.val)
            {
                SoftPhysicsHandler.colliderVisualizer.SyncPreviews();
            }
        }

        string ValueInfoFieldString(string groupName)
        {
            string result = "\n".Size(5);
            float value = Calc.RoundToDecimals(GetAverageGroupValue(groupName), _parameterGroup.roundFactor);
            switch(groupName)
            {
                case SoftVerticesGroupName.OUTER:
                {
                    result += "Outer (avg): ";
                    break;
                }
                case SoftVerticesGroupName.MAIN:
                {
                    result += "Main (avg): ";
                    break;
                }
                case SoftVerticesGroupName.AREOLA:
                {
                    result += "Areola (avg): ";
                    break;
                }
                case SoftVerticesGroupName.NIPPLE:
                {
                    result += "Nipple: ";
                    break;
                }
            }

            result += $"{value}".Bold();
            return result;
        }

        float GetAverageGroupValue(string groupName)
        {
            string paramName = _parameterGroup.name;
            switch(groupName)
            {
                case SoftVerticesGroupName.OUTER:
                {
                    var softJoints = SoftPhysicsHandler.outerSoftJoints[Side.LEFT];
                    return softJoints.Average(softJoint => softJoint.GetRealValue(paramName));
                }
                case SoftVerticesGroupName.MAIN:
                {
                    var softJoints = SoftPhysicsHandler.mainSoftJoints[Side.LEFT];
                    return softJoints.Average(softJoint => softJoint.GetRealValue(paramName));
                }
                case SoftVerticesGroupName.AREOLA:
                {
                    var softJoints = SoftPhysicsHandler.areolaSoftJoints[Side.LEFT];
                    return softJoints.Average(softJoint => softJoint.GetRealValue(paramName));
                }
                case SoftVerticesGroupName.NIPPLE: return SoftPhysicsHandler.nippleSoftJoints[Side.LEFT].GetRealValue(paramName);
                default: throw new Exception($"Invalid group name {groupName}");
            }
        }

        protected override void OnClose()
        {
            SoftPhysicsHandler.colliderVisualizer.ShowPreviewsJSON.val = false;
            SoftPhysicsHandler.softJointDiagnostics.DrawServiceSetEnabled(false);
        }
    }
}
